<?php

namespace Teknisa\DF\Modules\Saida\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\DF\Modules\Saida\Factories\ManifestoDocumentoEletronicoFactory;
use Teknisa\DF\Factories\TransmissaoNfeFactory;
use Teknisa\Libs\Util\Utilities;
use Teknisa\DF\Controllers\TeknisaCrud;
use NFePHP\Common\Keys;
use NFePHP\Common\Certificate;
use NFePHP\MDFe\Make;
use NFePHP\MDFe\Common\Standardize;
use NFePHP\MDFe\Tools;
use Teknisa\DF\Services\MDFe\Damdfe;

use Teknisa\Libs\Util\Environment;
    
class ManifestoDocumentoEletronicoService {

    protected $manifestoDocumentoEletronicoFactory;
    protected $environment;
    
    public function __construct(ManifestoDocumentoEletronicoFactory $manifestoDocumentoEletronicoFactory, TransmissaoNfeFactory $transmissaoNfeFactory, $fileServerLink, Environment $environment) {
        $this->manifestoDocumentoEletronicoFactory = $manifestoDocumentoEletronicoFactory;
        $this->transmissaoNfeFactory = $transmissaoNfeFactory;
        $this->fileServerLink = $fileServerLink;
        $this->environment = $environment;
        $this->pdfDir = sys_get_temp_dir();
    }
    
    /** 
     * Funções auxiliares para salvar registros 
     */
    
    /**
     * Retorna os dados padrão para o MDF-e de acordo com a empresa
     * 
     * @param string $cdEmpresa - Empresa
     * 
     * return array - Dados default do MDF-e
     */
    public function getDefaultMdfe($cdEmpresa) {
        return $this->manifestoDocumentoEletronicoFactory->getDefaultMdfe($cdEmpresa);
    }
    
    /**
     * Realiza a validação dos dados de um MDF-e
     * 
     * @param array $row - Dados do MDF-e
     * 
     * return boolean - Validade do MDF-e
     */
    public function validaMdfe($mdfe, &$erroMdfe) {
        $key = new Keys();
        $erroMdfe = "";
        $countNotas = 0;
        $estadosVizinhos = [
            "AC" => ["AM", "RO"],
            "AL" => ["PE", "SE", "BA"],
            "AP" => ["PA"],
            "AM" => ["RO", "RR", "AC", "PA", "MT"],
            "BA" => ["SE", "AL", "PE", "PI", "TO", "GO", "MG", "ES"],
            "CE" => ["RN", "PB", "PE", "PI"],
            "DF" => ["GO"],
            "ES" => ["RJ", "MG", "BA"],
            "GO" => ["TO", "MT", "BA", "MG", "MS", "DF"],
            "MA" => ["PI", "TO", "PA"],
            "MT" => ["RO", "AM", "PA", "TO", "GO", "MS"],
            "MS" => ["MT", "GO", "MG", "SP", "PR"],
            "MG" => ["GO", "BA", "ES", "RJ", "SP", "MS"],
            "PA" => ["AM", "RR", "AP", "MA", "TO", "MT"],
            "PB" => ["CE", "RN", "PE"],
            "PR" => ["MS", "SP", "SC"],
            "PE" => ["PI", "CE", "PB", "AL", "BA"],
            "PI" => ["MA", "CE", "PE", "BA", "TO"],
            "RJ" => ["MG", "ES", "SP"],
            "RN" => ["CE", "PB"],
            "RS" => ["SC"],
            "RO" => ["AC", "AM", "MT"],
            "RR" => ["AM", "PA"],
            "SC" => ["PR", "RS"],
            "SP" => ["MS", "MG", "RJ", "PR"],
            "SE" => ["BA", "AL"],
            "TO" => ["PA", "MA", "PI", "BA", "GO", "MT"]
        ];
        $estadoAnterior = "";
        
        if (!$mdfe["NOTAS_RELACIONADAS"]) {
            $erroMdfe .= "Não existem notas relacionadas.";
            $erroMdfe .= " ";
        }
        else {
            $sgDestino = $mdfe["NOTAS_RELACIONADAS"][0]["SGESTADODESTINONF"];
            foreach ($mdfe["NOTAS_RELACIONADAS"] as $nota) {
                if ($nota["SGESTADODESTINONF"] != $sgDestino) {
                    $erroMdfe .= "Existem notas com destinos em estados distintos.";
                    $erroMdfe .= " " ; 
                }
            }
        }
        if (!$mdfe["PERCURSO"]) {
            $erroMdfe .= "Não existe um percurso cadastrado.";
            $erroMdfe .= " ";
        }
        if (!$mdfe["MOTORISTAS"]) {
            $erroMdfe .= "Não existem motoristas cadastrados.";
            $erroMdfe .= " ";
        }
        if (!$key->isValid($mdfe["NRACESSOMDFE"])) {
            $erroMdfe .= "Chave de Acesso inválida.";
            $erroMdfe .= " ";
        }
        if (is_null($mdfe["VRCARGAMDFE"])) {
            $erroMdfe .= "Não existe Valor Total definido.";
            $erroMdfe .= " ";
        }
        if (is_null($mdfe["VRPESOBRUTOCARGA"])) {
            $erroMdfe .= "Não existe Peso Bruto definido.";
            $erroMdfe .= " ";
        }
        
        foreach ($mdfe["NOTAS_RELACIONADAS"] as $nota) {
            if (strlen($nota["NRACESSONFE"]) !== 44) {
                $countNotas++;
            }
        }
        if ($countNotas) {
            if ($countNotas == 1) {
                $erroMdfe .= $countNotas . " nota relacionada não possui Chave de Acesso.";
            }
            else {
                $erroMdfe .= $countNotas . " notas relacionadas não possuem Chave de Acesso."; 
            }
            $erroMdfe .= " ";
        }
        
        foreach ($mdfe["PERCURSO"] as $percurso) {
            if ($estadoAnterior) {
                if ($percurso["SGESTADO"] != $estadoAnterior) {
                    if (!in_array($percurso["SGESTADO"], $estadosVizinhos[$estadoAnterior])) {
                        $erroMdfe .= "Não existe um percurso direto entre " . $estadoAnterior . " e " . $percurso["SGESTADO"] . ".";
                        $erroMdfe .= " ";
                    }
                }
            }
            $estadoAnterior = $percurso["SGESTADO"];
        }
        
        if (!end($mdfe["PERCURSO"])["DESCARGA"]) {
            $erroMdfe .= "Não ocorre entrega no último destino do percurso.";
            $erroMdfe .= " ";
        }
        
        if ($erroMdfe) {
            return false;
        }
        return true;
    }
    
    /**
     * Insere o município de carregamento
     * 
     * @param array &$row - Row com os dados do MDF-e
     */
    public function insereCarregamentoMdfe($row) {
        $this->manifestoDocumentoEletronicoFactory->insereCarregamentoMdfe($row["CDVIATURA"], $row["NRLANCTOMDFE"], $row["CDPAIS"], $row["SGESTADO"], $row["CDMUNICIPIO"]);
    }
    
    /**
     * Seta o valor do campo NRSEQSEGCARMDFE que é o sequêncial para o Seguro de Carga
     * 
     * @param array &$row - Row com os dados do Seguro de Carga
     */
    public function setSequencialSeguroCarga(&$row) {
        $row["NRSEQSEGCARMDFE"] = str_pad($this->manifestoDocumentoEletronicoFactory->getSequencialSeguroCarga($row["CDVIATURA"], $row["NRLANCTOMDFE"]), 4, "0", STR_PAD_LEFT);
    }
    
    /**
     * Seta o valor do campo NRSEQAVERBAMDFE que é o sequêncial para a Averbação
     * 
     * @param array &$row - Row com os dados da Averbação
     */
    public function setSequencialAverbacao(&$row) {
        $row["NRSEQAVERBAMDFE"] = str_pad($this->manifestoDocumentoEletronicoFactory->getSequencialAverbacao($row["CDVIATURA"], $row["NRLANCTOMDFE"], $row["NRSEQSEGCARMDFE"]), 4, "0", STR_PAD_LEFT);        
    }
    
    /**
     * Seta o valor do campo NRSEQDADOSADIC que é o sequêncial para os Dados Adicionais do MDF-e
     * 
     * @param array &$row - Row com os dados dos Dados Adicionais
     */
    public function setSequencialDadosAdic(&$row) {
        $row["NRSEQDADOSADIC"] = str_pad($this->manifestoDocumentoEletronicoFactory->getSequencialDadosAdic($row["CDVIATURA"], $row["NRLANCTOMDFE"]), 4, "0", STR_PAD_LEFT);        
    }
    
    /**
     * Seta o valor dos campos NRSEQRELACNF e NRORDEMENTREGA que são os sequêncial para as Notas Relacionadas do MDF-e
     * 
     * @param array &$row - Row com os dados das Notas Relacionadas
     */
    public function setSequencialNotasRelac(&$row) {
        $sequencial = $this->manifestoDocumentoEletronicoFactory->getSequencialNotasRelac($row["CDVIATURA"], $row["NRLANCTOMDFE"]);
        $row["NRSEQRELACNF"] = $sequencial["NRSEQRELACNF"];
        $row["NRORDEMENTREGA"] = $sequencial["NRORDEMENTREGA"];
    }
    
    /**
     * Seta o valor dos campos NRSEQPERCURSO e NRORDEMENTREGA que são os sequêncial para o Percurso do MDF-e
     * 
     * @param array &$row - Row com os dados do Percurso
     */
    public function setSequencialPercurso(&$row) {
        $sequencial = $this->manifestoDocumentoEletronicoFactory->getSequencialPercurso($row["CDVIATURA"], $row["NRLANCTOMDFE"]);
        $row["NRSEQPERCURSO"] = str_pad($sequencial["NRSEQPERCURSO"], 4, "0", STR_PAD_LEFT);
        $row["NRORDEMENTREGA"] = str_pad($sequencial["NRORDEMENTREGA"], 3, "0", STR_PAD_LEFT);
    }
    
    /**
     * Seta o valor o campo NRMDFE que é o sequêncial para o MDF-e
     * 
     * @param array &$mdfe - Row com os dados do MDF-e
     */
    public function setSequencialNrmdfe(&$mdfe) {
        $sequencial = $this->manifestoDocumentoEletronicoFactory->setSequencialNrmdfe($mdfe["CDVIATURA"], $mdfe["NRLANCTOMDFE"], $mdfe["NRINSJURIEST"]);
        $mdfe["NRMDFE"] = $sequencial;
    }
    
    /**
     * Muda a ordem de entrega da Nota Relacionada
     * 
     * @param array $row - Row com os dados das Notas Relacionadas
     */
    public function mudarOrdemNota($row) {
        if ($row["SENTIDO"] == "1") {
            if ($row["NRORDEMENTREGA"] > 1) {
                $this->manifestoDocumentoEletronicoFactory->mudarOrdemNota($row["CDVIATURA"], $row["NRLANCTOMDFE"], $row["NRSEQRELACNF"], true);;
            }
        }
        else {
            $rowSequencial["CDVIATURA"] = $row["CDVIATURA"];
            $rowSequencial["NRLANCTOMDFE"] = $row["NRLANCTOMDFE"];
            $this->setSequencialNotasRelac($rowSequencial);
            if ($row["NRORDEMENTREGA"] < $rowSequencial["NRORDEMENTREGA"] - 1) {
                $this->manifestoDocumentoEletronicoFactory->mudarOrdemNota($row["CDVIATURA"], $row["NRLANCTOMDFE"], $row["NRSEQRELACNF"], false);;
            }
        }
    }
    
    /**
     * Muda a ordem de entrega do Percurso
     * 
     * @param array $row - Row com os dados do Percurso
     */
    public function mudarOrdemPercurso($row) {
        if ($row["SENTIDO"] == "1") {
            if ($row["NRORDEMENTREGA"] > 0) {
                $this->manifestoDocumentoEletronicoFactory->mudarOrdemPercurso($row["CDVIATURA"], $row["NRLANCTOMDFE"], $row["NRSEQPERCURSO"], true);;
            }
        }
        else {
            $rowSequencial["CDVIATURA"] = $row["CDVIATURA"];
            $rowSequencial["NRLANCTOMDFE"] = $row["NRLANCTOMDFE"];
            $this->setSequencialPercurso($rowSequencial);
            if ($row["NRORDEMENTREGA"] < $rowSequencial["NRORDEMENTREGA"] - 1) {
                $this->manifestoDocumentoEletronicoFactory->mudarOrdemPercurso($row["CDVIATURA"], $row["NRLANCTOMDFE"], $row["NRSEQPERCURSO"], false);;
            }
        }
    }
    
    /**
     * Exclui a Nota Relacionada, realizando a mudança do campo NRORDEMENTREGA das outras notas
     * 
     * @param array $row - Row com os dados das Notas Relacionadas
     */
    public function excluirNota($row) {
        $this->manifestoDocumentoEletronicoFactory->excluirNota($row["CDVIATURA"], $row["NRLANCTOMDFE"], $row["NRSEQRELACNF"]);
        $this->atualizaDadosNotas($row);
    }
    
    /**
     * Exclui o Percurso, realizando a mudança do campo NRORDEMENTREGA dos outros destinos
     * 
     * @param array $row - Row com os dados do Percurso
     */
    public function excluirPercurso($row) {
        $this->manifestoDocumentoEletronicoFactory->excluirPercurso($row["CDVIATURA"], $row["NRLANCTOMDFE"], $row["NRSEQPERCURSO"]);
    }
    
    /**
     * Limpa o Percurso de um MDF-e
     * 
     * @param array $row - Row com os dados do MDF-e
     */
    public function limparPercurso($row) {
        $this->manifestoDocumentoEletronicoFactory->limparPercurso($row["CDVIATURA"], $row["NRLANCTOMDFE"]);
    }
    
    /**
     * Atualiza os dados da MDF-e para corresponder aos dados das notas relacionadas. Calcula o Valor Total e Peso Bruto do MDF-e
     * 
     * @param array $row - Row com os dados do MDF-e
     */
    public function atualizaDadosNotas(&$row) {
        $dados = $this->manifestoDocumentoEletronicoFactory->atualizaDadosNotas($row["CDVIATURA"], $row["NRLANCTOMDFE"]);
        $row["VRPESOBRUTOCARGA"] = $dados["VRPESOBRUTOCARGA"];
        $row["VRCARGAMDFE"] = $dados["VRCARGAMDFE"];
    }
    
    /**
     * Atualiza o percurso, inserindo os municípios de entrega das notas.
     * 
     * @param array $row - Row com os dados da nota.
     */
    public function atualizaPercurso(&$row) {
        $row["__is_new"] = true;
        $row["CDPAIS"] = '0055';
        $row["SGESTADO"] = $row["SGESTADODESTINONF"];
        $row["CDMUNICIPIO"] = $row["CDMUNICIPIODESTINONF"];
        return $this->manifestoDocumentoEletronicoFactory->verificaPercurso($row["CDVIATURA"], $row["NRLANCTOMDFE"], $row["SGESTADODESTINONF"], $row["CDMUNICIPIODESTINONF"]); 
    }
    
    /**
     * Limpa os dados de um MDF-e, removendo as notas relacionadas, percurso, motoristas, reboques, seguro de carga, averbacao e dados adicionais.
     * 
     * @param array $row - Row com os dados do MDF-e.
     */
    public function limparDadosMdfe($row) {
        $this->manifestoDocumentoEletronicoFactory->limparNota($row["CDVIATURA"], $row["NRLANCTOMDFE"]);
        $this->manifestoDocumentoEletronicoFactory->limparPercurso($row["CDVIATURA"], $row["NRLANCTOMDFE"], true);
        $this->manifestoDocumentoEletronicoFactory->limparMotorista($row["CDVIATURA"], $row["NRLANCTOMDFE"]);
        $this->manifestoDocumentoEletronicoFactory->limparReboque($row["CDVIATURA"], $row["NRLANCTOMDFE"]);
        $this->manifestoDocumentoEletronicoFactory->limparAverbacao($row["CDVIATURA"], $row["NRLANCTOMDFE"]);
        $this->manifestoDocumentoEletronicoFactory->limparSeguroCarga($row["CDVIATURA"], $row["NRLANCTOMDFE"]);
        $this->manifestoDocumentoEletronicoFactory->limparDadosAdic($row["CDVIATURA"], $row["NRLANCTOMDFE"]);
    }
    
    public function excluiMDFE($row){
        $this->manifestoDocumentoEletronicoFactory->excluiMDFE($row["CDVIATURA"], $row["NRLANCTOMDFE"]);
    }
    
    /**
     * Seta o novo status do MDF-e
     * 
     * @param array $mdfe - Row com os dados do MDF-e    string $status - Novo status do MDF-e
     */
    public function setStatusMdfe($mdfe, $status) {
        $sequencial = $this->manifestoDocumentoEletronicoFactory->setStatusMdfe($mdfe["CDVIATURA"], $mdfe["NRLANCTOMDFE"], $status);
    }
    
     /**
     * Cria e salva a chave de acesso para realizar a transmissão de um MDF-e
     * 
     * @param string $mdfe - Objeto contendo os dados do MDF-e
     */
    public function setChaveAcessoMdfe(&$mdfe) {
        $key = new Keys();
        
        $chaveAcesso = $key->build($mdfe["CDESTADOIBGE"], date('y'), date('m'), $mdfe["NRINSJURIEST"], 
                                   str_pad($mdfe["CDMODDOCFISCMDFE"], 2, "0", STR_PAD_LEFT), 
                                   str_pad($mdfe["CDSERIEMDFE"], 3, "0", STR_PAD_LEFT),
                                   str_pad($mdfe["NRMDFE"], 9, "0", STR_PAD_LEFT), "1",
                                   str_pad(substr($mdfe["NRLANCTOMDFE"], -8), 8, "0", STR_PAD_LEFT));
        
        $this->manifestoDocumentoEletronicoFactory->setChaveAcessoMdfe($mdfe["CDVIATURA"], $mdfe["NRLANCTOMDFE"], $chaveAcesso);                           
        $mdfe["NRACESSOMDFE"] = $chaveAcesso;
    }
    
    /**
     * Cria um XML para realizar a transmissão de um MDF-e
     * 
     * @param string $mdfe - Objeto contendo os dados do MDF-e
     * 
     * return array - Dados default do MDF-e
     */
    public function criaXmlTransmissaoMdfe($mdfe) {
        $makeMdfe = new Make();
        $makeMdfe->setOnlyAscii(true);
        
        /*
         * Grupo ide ( Identificação )
         */
        $std = new \stdClass();
        $std->cUF = $mdfe["CDESTADOIBGE"];
        $std->tpAmb = $mdfe["IDTPAMBIENTEMDFE"];
        if ($mdfe["IDPRESTSERVTRANSP"] == "S") {
            $std->tpEmit = '1'; //Prestador de serviço de transporte
        }
        else {
            $std->tpEmit = '2'; //Não prestador de serviço de transporte
        }
        $std->mod = $mdfe["CDMODDOCFISCMDFE"];
        $std->serie = ltrim($mdfe["CDSERIEMDFE"], '0');
        $std->nMDF = ltrim($mdfe["NRMDFE"], '0');
        $std->cMDF = substr($mdfe["NRLANCTOMDFE"], -8);
        $std->cDV = substr($mdfe["NRACESSOMDFE"], -1);
        $std->modal = '1'; //1: Rodoviário
                           //2: Aéreo
                           //3: Aquaviário
                           //4: Ferroviário
        $std->dhEmi = date("Y-m-d") . 'T' . date("H:i:s") . date("P");
        $std->tpEmis = '1'; //1: Normal
                            //2: Contingência
        $std->procEmi = '0';  //0: Emissao de MDF-e com aplicativo do contribuinte
                              //3: Emissao de MDF-e pelo contribuinte com aplicativo fornecido pelo Fisco
        $std->verProc = 'TEK_EXP_MOBILE';
        $std->UFIni = $mdfe["PERCURSO"][0]["SGESTADO"];
        $std->UFFim = end($mdfe["PERCURSO"])["SGESTADO"];
        //$std->dhIniViagem = $mdfe["DTMDFE"];
        $makeMdfe->tagide($std);
        
        $infMunCarrega = new \stdClass();
        $infMunCarrega->cMunCarrega = $mdfe["PERCURSO"][0]["CDMUNICIBGE"];
        $infMunCarrega->xMunCarrega = $mdfe["PERCURSO"][0]["NMMUNICIPIO"];
        $makeMdfe->taginfMunCarrega($infMunCarrega);
        
        $sgEntrega = end($mdfe["PERCURSO"])["SGESTADO"];
        $ultimoEstado = '';
        foreach ($mdfe["PERCURSO"] as $percurso) {
            if ($percurso["NRORDEMENTREGA"] == 0 || $percurso["SGESTADO"] == $sgEntrega) {
                continue;
            }
            if ($percurso["SGESTADO"] != $ultimoEstado) {
                $infPercurso = new \stdClass();
                $infPercurso->UFPer = $percurso["SGESTADO"];
                $makeMdfe->taginfPercurso($infPercurso);
                $ultimoEstado = $percurso["SGESTADO"];
            }
        }
        
        /*
         * fim ide
         */
        
        /*
         * Grupo emit ( Emitente )
         */
        $std = new \stdClass();
        $std->CNPJ = $mdfe["NRINSJURIEST"];
        $std->IE = $mdfe["CDINSCESTA"];
        $std->xNome = $mdfe["NMRAZSOCEMPR"];
        $std->xFant = $mdfe["NMRAZSOCEMPR"];
        $makeMdfe->tagemit($std);
        
        $std = new \stdClass();
        $std->xLgr = $mdfe["DSENDEIEST"];
        $std->nro = '.';
        if ($mdfe["DSCOMPENDIES"]) {
            $std->xCpl = $mdfe["DSCOMPENDIES"];
        }
        $std->xBairro = $mdfe["NMBAIRIEST"];
        $std->cMun = $mdfe["CDMUNICIBGE"];
        $std->xMun = $mdfe["NMMUNICIPIO"];
        if ($mdfe["NRCEPIEST"]) {
            $std->CEP = $mdfe["NRCEPIEST"]; 
        }
        $std->UF = $mdfe["SGESTADO"];
        if ($mdfe["NRTELEIEST"]) {
            $std->fone = $mdfe["NRTELEIEST"]; 
        }
        if (strlen($mdfe["IDEMAILMDFE"]) >= 6) {
            $std->email = $mdfe["IDEMAILMDFE"]; 
        }
        $makeMdfe->tagenderEmit($std);
        /*
         * fim emit
         */
         
        /*
         * Grupo rodo ( Rodoviário )
         */
        
        /* Grupo infANTT */
        if ($mdfe["VIATURA"]["NRRNTRCVEIC"]) {
           $infANTT = new \stdClass();
            $infANTT->RNTRC = $mdfe["VIATURA"]["NRRNTRCVEIC"];
            $makeMdfe->taginfANTT($infANTT);
        }
        /*
        /* informações do CIOT */
        if ($mdfe["NRCIOT"]) {
            $infCIOT = new \stdClass();
            $infCIOT->CIOT = $mdfe["NRCIOT"];
            if (strlen($mdfe["NRINSJURRESPGER"]) == 14) {
                $infCIOT->CNPJ = $mdfe["NRINSJURRESPGER"]; 
            }
            else {
                $infCIOT->CPF = $mdfe["NRINSJURRESPGER"];
            }
            $makeMdfe->taginfCIOT($infCIOT);
        }
        
        /* fim infANTT */
        
        /* Grupo veicTracao */
        $veicTracao = new \stdClass();
        $veicTracao->cInt = $mdfe["VIATURA"]["CDVIATURA"];
        $veicTracao->placa = $mdfe["VIATURA"]["NRPLACA"];
        $veicTracao->tara = $mdfe["VIATURA"]["QTTARAVIAT"];
        if ($mdfe["VIATURA"]["QTCAPACIVIAT"]) {
           $veicTracao->capKG = $mdfe["VIATURA"]["QTCAPACIVIAT"]; 
        }
        $veicTracao->tpRod = $mdfe["VIATURA"]["IDTPRODADO"];
        $veicTracao->tpCar = $mdfe["VIATURA"]["IDTPCARROCERIA"];
        $veicTracao->UF = $mdfe["VIATURA"]["SGUFVEICULO"];
        
        $veicTracao->condutor = array();
        foreach ($mdfe["MOTORISTAS"] as $motorista) {
            $condutor = new \stdClass();
            $condutor->xNome = $motorista["NMMOTORISTA"];
            $condutor->CPF = $motorista["NRCPF"];
            array_push($veicTracao->condutor, $condutor); 
        }
        
        /*
            Foi comentado este bloco pois segundo a especificação
            a rotina de MDF-e desenvolvida pelo Sistema Teknisa seria
            para atender apenas empresas que transportam suas próprias
            mercadorias, ou seja, o veiculo utilizado para o transporte
            não está atrelado a nenhuma transportadora
        */
        /*
        if ($mdfe["VIATURA"]["NRINSJURTRAN"]) {
            $prop = new \stdClass();
            if ($mdfe["VIATURA"]["IDTPIJURTRAN"] == 'F') {
                $prop->CPF = $mdfe["VIATURA"]["NRINSJURTRAN"]; 
            }
            else {
                $prop->CNPJ = $mdfe["VIATURA"]["NRINSJURTRAN"]; 
            }
            $prop->RNTRC = $mdfe["VIATURA"]["NRRNTRC"];
            $prop->xNome = $mdfe["VIATURA"]["NMRAZSOCTRAN"];
            $prop->IE = $mdfe["VIATURA"]["NRINSESTTRAN"];
            $prop->UF = $mdfe["VIATURA"]["SGUFVEICULO"];
            $prop->tpProp = $mdfe["VIATURA"]["IDTPPROPRIETA"];
            $veicTracao->prop = $prop;
        }
        */
        
        $makeMdfe->tagveicTracao($veicTracao);
        
        /* fim veicTracao */
        
        /* Grupo veicReboque */
        foreach ($mdfe["REBOQUES"] as $reboque) {
            $veicReboque = new \stdClass();
            $veicReboque->cInt = $reboque["VIATURA"]["CDVIATURA"];
            $veicReboque->placa = $reboque["VIATURA"]["NRPLACA"];
            if ($reboque["VIATURA"]["QTTARAVIAT"]) {
                $veicReboque->tara = $reboque["VIATURA"]["QTTARAVIAT"];
            }
            if ($reboque["VIATURA"]["QTCAPACIVIAT"]) {
                $veicReboque->capKG = $reboque["VIATURA"]["QTCAPACIVIAT"];
            }
            $veicReboque->tpCar = $reboque["VIATURA"]["IDTPCARROCERIA"];
            $veicReboque->UF = $reboque["VIATURA"]["SGUFVEICULO"];

            $makeMdfe->tagveicReboque($veicReboque); 
        }
        
        /* fim veicReboque */
        
        /* fim rodo */
        
        /*
         * Grupo infDoc ( Documentos fiscais )
         */
        
        $nItemPercurso = 0;
        foreach($mdfe["PERCURSO"] as $percurso) {
            if ($percurso["NRORDEMENTREGA"] == 0 || !$percurso["DESCARGA"]) {
                continue;
            }
            
            $infMunDescarga = new \stdClass();
            $infMunDescarga->cMunDescarga = $percurso["CDMUNICIBGE"];
            $infMunDescarga->xMunDescarga = $percurso["NMMUNICIPIO"];
            $infMunDescarga->nItem = $nItemPercurso;
            $makeMdfe->taginfMunDescarga($infMunDescarga);
            
            
            foreach($mdfe["NOTAS_RELACIONADAS"] as $nota) {
                if ($nota["CDMUNICIPIODESTINONF"] == $percurso["CDMUNICIPIO"] && $nota["SGESTADODESTINONF"] == $percurso["SGESTADO"]) {
                    /* infNFe */
                    if ($nota["IDTPDOCUMENTO"] == '0') {
                        $std = new \stdClass();
                        $std->chNFe = $nota["NRACESSONFE"];
                        $std->nItem = $nItemPercurso;
                        $makeMdfe->taginfNFe($std);
                    }
                    /* infCTe */
                    else {
                        $std = new \stdClass();
                        $std->chCTe = $nota["NRACESSONFE"];
                        $std->nItem = $nItemPercurso;
                        $makeMdfe->taginfCTe($std);
                    }
                }
            }
            
            $nItemPercurso++;
            
        }
        
        /* fim grupo infDoc */
        
        /* Grupo do Seguro */
        foreach($mdfe["SEGURO_CARGA"] as $seguro) {
            $std = new \stdClass();
            $std->respSeg = $seguro["IDRESPSEGURO"];
            if ($seguro["IDTPINSJURRESP"] == 'J') {
                $std->CNPJ = $seguro["NRINSJURRESP"];
            }
            else if ($seguro["IDTPINSJURRESP"] == 'F') {
                $std->CPF = $seguro["NRINSJURRESP"];
            }
            
            /* Informações da seguradora */
            $stdinfSeg = new \stdClass();
            $stdinfSeg->xSeg = $seguro["NMSEGURADORA"];
            $stdinfSeg->CNPJ = $seguro["NRINSCSEGURADORA"];
            
            $std->infSeg = $stdinfSeg;
            
            if ($seguro["NRAPOLISE"]) {
                $std->nApol = $seguro["NRAPOLISE"];
            }
            
            $nAver = array();
            foreach ($seguro["AVERBACAO"] as $averbacao) {
                array_push($nAver, $averbacao["NRAVERBACAO"]);
            }
            $std->nAver = $nAver;
            $makeMdfe->tagseg($std);
        }
        /* fim grupo Seguro */
        
        /* grupo de totais */
        $std = new \stdClass();
        $std->vCarga = number_format($mdfe["VRCARGAMDFE"], 2 ,'.' , '');
        $std->cUnid = str_pad($mdfe["IDUNIDADECARGA"], 2, "0", STR_PAD_LEFT);
        $std->qCarga = number_format($mdfe["VRPESOBRUTOCARGA"], 4 ,'.' , '');
        $makeMdfe->tagtot($std);
        /* fim grupo de totais */
        
        /* grupo Informações Adicionais */
        $dadosAdicionais = "";
        foreach ($mdfe["DADOS_ADICIONAIS"] as $dado_adicional) {
            $dadosAdicionais .= $dado_adicional["DSDADOADIC"] . ". ";
        }
        $std = new \stdClass();
        $std->infCpl = $dadosAdicionais;
        $std->infAdFisco = $dadosAdicionais;
        $makeMdfe->taginfAdic($std);
        
        /* fim grupo Informações Adicionais */
        
        try {
            $xml = $makeMdfe->getXML(); 
        }
        catch (\Exception $e) {
            $erros = $makeMdfe->getErrors();
            
            $mensagemErro = '';
            foreach ($erros as $erro) {
                $mensagemErro .= $erro .= ".\n";
            }
            throw new \Exception ($mensagemErro, 10);
        }
        
        return $xml;
    }
    
    /**
     * Salva o xml no banco
     * 
     * @param array $mdfe - dados do Mdfe     string $xml - xml a ser salvo
     */
    public function salvaXml($mdfe, $idtipoarqxmlmdfe, $xml) {
        $this->manifestoDocumentoEletronicoFactory->salvaXml($mdfe["CDVIATURA"], $mdfe["NRLANCTOMDFE"], $idtipoarqxmlmdfe, $xml);
    }
    
     /**
     * Realiza a transmissão de um MDF-e
     * 
     * @param string $xml - xml a ser transmitido   string $cdEmpresa - Empresa   array $config - Configuração
     * 
     * return object - objeto com status atual do mdfe
     */
    public function transmiteMdfe($xml, $cdempresa, $idLote, $config) {
        $certificado = $this->getCertificado($cdempresa, $config);
        
        $tools = new Tools(json_encode($config), $certificado);
        $xmlAssinado = $tools->signMDFe($xml);
        
        $resp = $tools->sefazEnviaLote([$xmlAssinado], $idLote);
        
        $st = new Standardize();
        $std = $st->toStd($resp);
        
        $nRec = $std->infRec->nRec;
        
        sleep(3);

        $resp = $tools->sefazConsultaRecibo($nRec);
        $std = ($st->toStd($resp))->protMDFe->infProt;
        
        return $std;
    }
    
    /**
     * Realiza o cancelamento de um MDF-e
     * 
     * @param string $chaveAcesso - chave de acesso do MDF-e   string $cdEmpresa - Empresa   array $config - Configuração
     * 
     * return object - objeto com status atual do mdfe
     */
    public function cancelaMdfe($chaveAcesso, $cdempresa, $config, $motivoCancelamento) {
        $certificado = $this->getCertificado($cdempresa, $config);
        $consultaChave = $this->consultaChave($chaveAcesso, $cdempresa, $config);
        
        $tools = new Tools(json_encode($config), $certificado);
        
        $xJust = $motivoCancelamento;
        $nProt = $consultaChave->protMDFe->infProt->nProt;
        $resp = $tools->sefazCancela($chaveAcesso, $xJust, $nProt);
        
        $st = new Standardize();
        $std = $st->toStd($resp);

        return $std->infEvento;
    }
    
    /**
     * Realiza o encerramento de um MDF-e
     * 
     * @param string $chaveAcesso - chave de acesso do MDF-e   string $cdEmpresa - Empresa   array $config - Configuração
     * 
     * return object - objeto com status atual do mdfe
     */
    public function encerraMdfe($chaveAcesso, $cdempresa, $cUF, $cMun, $config) {
        $certificado = $this->getCertificado($cdempresa, $config);
        $consultaChave = $this->consultaChave($chaveAcesso, $cdempresa, $config);
        
        $tools = new Tools(json_encode($config), $certificado);
        
        $nProt = $consultaChave->protMDFe->infProt->nProt;
        $dtEnc = date("Y-m-d"); // Opcional, caso nao seja preenchido pegara HOJE
        
        $resp = $tools->sefazEncerra($chaveAcesso, $nProt, $cUF, $cMun, $dtEnc);
        
        $st = new Standardize();
        $std = $st->toStd($resp);
        
        return $std->infEvento;
    }
    
    /**
     * Realiza a consulta de um MDF-e por meio de sua chave de acesso
     * 
     * @param string $chaveAcesso - chave de acesso do MDF-e   string $cdEmpresa - Empresa   array $config - Configuração
     * 
     * return object $consultaChave - status e outros dados do MDF-e
     */
    public function consultaChave ($chaveAcesso, $cdempresa, $config) {
        $certificado = $this->getCertificado($cdempresa, $config);
        
        $tools = new Tools(json_encode($config), $certificado);

        $consulta = $tools->sefazConsultaChave($chaveAcesso);
        $st = new Standardize();
        $consultaChave = $st->toStd($consulta);
        return $consultaChave;
    }
    
    /**
     * Realiza a criação de um DAMDFE
     * 
     * @param string $xml - xml do MDF-e
     * 
     * return string $pdfDamdfe - Caminho do pdf gerado
     */
    public function downloadDamdfe ($xml, $chaveAcesso) {
        $pdfDamdfe = $this->pdfDir . DIRECTORY_SEPARATOR . "{$chaveAcesso}-nfe.pdf";
        $logo = '';
        $this->salvaDamdfe($xml, $pdfDamdfe, $logo);
        return $pdfDamdfe;
    }
    
    /**
     * Salva o DAMDFE
     * 
     * @param string $xmlProt - xml do MDF-e   string $pdfDamdfe - Caminho para ser salvo   string $logo - logo (opcional)
     * 
     * return object $damdfe - Objeto Damdfe gerado
     */
    private function salvaDamdfe($xmlProt, $pdfDamdfe, $logo = ''){
        $damdfe = new Damdfe($xmlProt, 'P', 'A4',  $logo, 'I', '');
        $pdf = $damdfe->render($logo);
        
        $output = fopen($pdfDamdfe, "w"); //Salva o PDF na pasta
        fwrite($output, $pdf);
        fclose($output);
        
        return $damdfe;
    }
    
    public function decrypt($text, $salt = 'teknisa') {
          return Utilities::zeedhiDecrypt($text, $salt);
    }
    
    /**
     * Obtem o certificado da Empresa
     * 
     * @param string $cdEmpresa - Empresa   array $config - Configuração
     * 
     * return object - Certificado
     */
    public function getCertificado($cdempresa, $config) {
        $certificado = $this->transmissaoNfeFactory->getCertificado($config["ie"], $config["cnpj"], $cdempresa);

		$certificadoEncontrado = false;
		
		foreach($certificado as $cert) {
		    if($cert["DSDIRCERTIFICADO"]) {
		        $certificadoEncontrado = true;
		        $certify = $cert;
		    }
		}
		
		if($certificadoEncontrado == false) {
		    throw new \Exception("Certificado Digital não encontrado.");
		}
		
        $senha = $this->decrypt($certify["CDSENHACERTIF"]);

		$certificado = $certify["DSDIRCERTIFICADO"];
		
		$certificado = explode('>', $certificado);
		
        $certificado = file_get_contents($this->fileServerLink.$certificado[1]);
        
        try {
            $certificado = Certificate::readPfx($certificado, $senha);
            $this->certificate = $certificado;
        }
        catch(\Exception $e) {
            throw new \Exception("Senha do certificado incorreta!");
        }
        
        return $certificado;
    }

}
